@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - http://www.xibo.org.uk
  *
@@ -557,7 +557,8 @@ class Library extends Base
             'layoutId' => $parsedQueryParams->getInt('layoutId'),
             'includeLayoutBackgroundImage' => ($parsedQueryParams->getInt('layoutId') != null) ? 1 : 0,
             'orientation' => $parsedQueryParams->getString('orientation', ['defaultOnEmptyString' => true]),
-            'logicalOperator' => $parsedQueryParams->getString('logicalOperator')
+            'logicalOperator' => $parsedQueryParams->getString('logicalOperator'),
+            'logicalOperatorName' => $parsedQueryParams->getString('logicalOperatorName'),
         ], $parsedQueryParams));
 
         // Add some additional row content
@@ -979,7 +980,7 @@ class Library extends Base
      *  operationId="libraryAdd",
      *  tags={"library"},
      *  summary="Add Media",
-     *  description="Add Media to the Library",
+     *  description="Add Media to the Library, optionally replacing an existing media item, optionally adding to a playlist.",
      *  @SWG\Parameter(
      *      name="files",
      *      in="formData",
@@ -1030,16 +1031,23 @@ class Library extends Base
      *      required=false
      *   ),
      *  @SWG\Parameter(
+     *      name="playlistId",
+     *      in="formData",
+     *      description="A playlistId to add this uploaded media to",
+     *      type="integer",
+     *      required=false
+     *   ),
+     *  @SWG\Parameter(
      *      name="widgetFromDt",
      *      in="formData",
-     *      description="Date in Y-m-d H:i:s format, will set widget start date",
+     *      description="Date in Y-m-d H:i:s format, will set widget start date. Requires a playlistId.",
      *      type="string",
      *      required=false
      *   ),
      *  @SWG\Parameter(
      *      name="widgetToDt",
      *      in="formData",
-     *      description="Date in Y-m-d H:i:s format, will set widget end date",
+     *      description="Date in Y-m-d H:i:s format, will set widget end date. Requires a playlistId.",
      *      type="string",
      *      required=false
      *   ),
@@ -1074,7 +1082,17 @@ class Library extends Base
     {
         $parsedBody = $this->getSanitizer($request->getParams());
         $options = $parsedBody->getArray('options', ['default' => []]);
-        $oldFolderId = 1;
+
+        // Folders
+        $folderId = $parsedBody->getInt('folderId');
+
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
+        }
+
+        if (empty($folderId) || !$this->getUser()->featureEnabled('folder.view')) {
+            $folderId = $this->getUser()->homeFolderId;
+        }
 
         $options = array_merge([
             'oldMediaId' => null,
@@ -1138,7 +1156,7 @@ class Library extends Base
             'widgetFromDt' => $widgetFromDt === null ? null : $widgetFromDt->format('U'),
             'widgetToDt' => $widgetToDt === null ? null : $widgetToDt->format('U'),
             'deleteOnExpiry' => $parsedBody->getCheckbox('deleteOnExpiry', ['checkboxReturnInteger' => true]),
-            'oldFolderId' => $parsedBody->getInt('folderId', ['default' => $oldFolderId]),
+            'oldFolderId' => $folderId,
             'routeParser' => RouteContext::fromRequest($request)->getRouteParser()
         ];
 
@@ -1295,6 +1313,9 @@ class Library extends Base
         $media->orientation = $sanitizedParams->getString('orientation', ['default' => $media->orientation]);
 
         if ($media->hasPropertyChanged('folderId')) {
+            if ($media->folderId === 1) {
+                $this->checkRootFolderAllowSave();
+            }
             $folder = $this->folderFactory->getById($media->folderId);
             $media->permissionsFolderId = ($folder->getPermissionFolderId() == null)
                 ? $folder->id
@@ -1995,7 +2016,14 @@ class Library extends Base
             throw new AccessDeniedException();
         }
 
-        $layouts = $this->layoutFactory->query(null, ['mediaId' => $id, 'showDrafts' => 1]);
+        $sanitizedParams = $this->getSanitizer($request->getParams());
+        $layouts = $this->layoutFactory->query(
+            $this->gridRenderSort($sanitizedParams),
+            $this->gridRenderFilter([
+                'mediaId' => $id,
+                'showDrafts' => 1
+            ], $sanitizedParams)
+        );
 
         if (!$this->isApi($request)) {
             foreach ($layouts as $layout) {
@@ -2329,16 +2357,18 @@ class Library extends Base
             'default' => $this->getConfig()->getSetting('MEDIA_STATS_ENABLED_DEFAULT')
         ]);
 
-        $folderId = $sanitizedParams->getInt('folderId', ['default' => 1]);
-        if ($this->getUser()->featureEnabled('folder.view')) {
-            $folder = $this->folderFactory->getById($folderId);
-            $permissionsFolderId = ($folder->permissionsFolderId == null)
-                ? $folder->id
-                : $folder->permissionsFolderId;
-        } else {
-            $permissionsFolderId = 1;
+        // Folders
+        $folderId = $sanitizedParams->getInt('folderId');
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
         }
 
+        if (empty($folderId) || !$this->getUser()->featureEnabled('folder.view')) {
+            $folderId = $this->getUser()->homeFolderId;
+        }
+
+        $folder = $this->folderFactory->getById($folderId, 0);
+
         if ($sanitizedParams->hasParam('expires')) {
             if ($sanitizedParams->getDate('expires')->format('U') > Carbon::now()->format('U')) {
                 $expires = $sanitizedParams->getDate('expires')->format('U');
@@ -2403,8 +2433,8 @@ class Library extends Base
                 'duration' => $module->determineDuration(),
                 'extension' => $ext,
                 'enableStat' => $enableStat,
-                'folderId' => $folderId,
-                'permissionsFolderId' => $permissionsFolderId
+                'folderId' => $folder->getId(),
+                'permissionsFolderId' => $folder->getPermissionFolderIdOrThis()
             ]
         );
 
@@ -2478,7 +2508,9 @@ class Library extends Base
             throw new InvalidArgumentException(__('Invalid image data'));
         }
 
-        $media->orientation = ($image->getWidth() >= $image->getHeight()) ? 'landscape' : 'portrait';
+        $media->width = $image->getWidth();
+        $media->height = $image->getHeight();
+        $media->orientation = ($media->width >= $media->height) ? 'landscape' : 'portrait';
         $media->save(['saveTags' => false, 'validate' => false]);
 
         return $response->withStatus(204);
@@ -2566,6 +2598,9 @@ class Library extends Base
         }
 
         $folderId = $this->getSanitizer($request->getParams())->getInt('folderId');
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
+        }
 
         $media->folderId = $folderId;
         $folder = $this->folderFactory->getById($media->folderId);
@@ -2597,15 +2632,15 @@ class Library extends Base
     {
         $params = $this->getSanitizer($request->getParams());
         $items = $params->getArray('items');
-        $folderId = $params->getInt('folderId', ['default' => 1]);
-        if ($this->getUser()->featureEnabled('folder.view')) {
-            $folder = $this->folderFactory->getById($folderId);
-            $permissionsFolderId = ($folder->permissionsFolderId == null)
-                ? $folder->id
-                : $folder->permissionsFolderId;
-        } else {
-            $permissionsFolderId = 1;
+
+        // Folders
+        $folderId = $params->getInt('folderId');
+        if (empty($folderId) || !$this->getUser()->featureEnabled('folder.view')) {
+            $folderId = $this->getUser()->homeFolderId;
         }
+        $folder = $this->folderFactory->getById($folderId, 0);
+
+        // Stats
         $enableStat = $params->getString('enableStat', [
             'default' => $this->getConfig()->getSetting('MEDIA_STATS_ENABLED_DEFAULT')
         ]);
@@ -2656,8 +2691,8 @@ class Library extends Base
                             'fileType' => strtolower($module->getModuleType()),
                             'duration' => !(empty($import->searchResult->duration)) ? $import->searchResult->duration : $module->determineDuration(),
                             'enableStat' => $enableStat,
-                            'folderId' => $folderId,
-                            'permissionsFolderId' => $permissionsFolderId
+                            'folderId' => $folder->getId(),
+                            'permissionsFolderId' => $folder->permissionsFolderId
                         ]
                     );
                 } else {
@@ -2691,7 +2726,7 @@ class Library extends Base
                             );
 
                             list($imgWidth, $imgHeight) = @getimagesize($filePath);
-                            $media->updateOrientation(($imgWidth >= $imgHeight) ? 'landscape' : 'portrait');
+                            $media->updateOrientation($imgWidth, $imgHeight);
                         } catch (\Exception $exception) {
                             // if we failed, corrupted file might still be created, remove it here
                             unlink($libraryLocation . $media->getId() . '_' . $media->mediaType . 'cover.png');
