@@ -1,8 +1,8 @@
 <?php
-/**
- * Copyright (C) 2021 Xibo Signage Ltd
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -282,7 +282,8 @@ class User extends Base
             'userTypeId' => $sanitizedParams->getInt('userTypeId'),
             'userName' => $sanitizedParams->getString('userName'),
             'useRegexForName' => $sanitizedParams->getCheckbox('useRegexForName'),
-            'retired' => $sanitizedParams->getInt('retired')
+            'retired' => $sanitizedParams->getInt('retired'),
+            'logicalOperatorName' => $sanitizedParams->getString('logicalOperatorName'),
         ];
 
         // Load results into an array
@@ -324,6 +325,9 @@ class User extends Base
                 $user->homePage = __('Unknown homepage, please edit to update.');
             }
 
+            // Set the home folder
+            $user->homeFolder = $user->getUnmatchedProperty('homeFolder', '/');
+
             // Super admins have some buttons
             if ($this->getUser()->featureEnabled('users.modify')
                 && $this->getUser()->checkEditable($user)
@@ -350,6 +354,23 @@ class User extends Base
                 ];
             }
 
+            if ($this->getUser()->featureEnabled('folder.userHome')) {
+                $user->buttons[] = [
+                    'id' => 'user_button_set_home',
+                    'url' => $this->urlFor($request, 'user.homeFolder.form', ['id' => $user->userId]),
+                    'text' => __('Set Home Folder'),
+                    'multi-select' => true,
+                    'dataAttributes' => [
+                        ['name' => 'commit-url', 'value' => $this->urlFor($request, 'user.homeFolder', ['id' => $user->userId])],
+                        ['name' => 'commit-method', 'value' => 'post'],
+                        ['name' => 'id', 'value' => 'user_button_set_home'],
+                        ['name' => 'text', 'value' => __('Set home folder')],
+                        ['name' => 'rowtitle', 'value' => $user->userName],
+                        ['name' => 'form-callback', 'value' => 'userHomeFolderMultiselectFormOpen']
+                    ],
+                ];
+            }
+
             if ($this->getUser()->featureEnabled('users.modify')
                 && $this->getUser()->checkPermissionsModifyable($user)
             ) {
@@ -557,6 +578,13 @@ class User extends Base
         $user->libraryQuota = $sanitizedParams->getInt('libraryQuota', ['default' => 0]);
         $user->setNewPassword($sanitizedParams->getString('password'));
 
+        // Are user home folders enabled? If not, use the default.
+        if ($this->getUser()->featureEnabled('folder.userHome')) {
+            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId');
+        } else {
+            $user->homeFolderId = 1;
+        }
+
         if ($this->getUser()->isSuperAdmin()) {
             $user->userTypeId = $sanitizedParams->getInt('userTypeId');
             $user->isSystemNotification = $sanitizedParams->getCheckbox('isSystemNotification');
@@ -805,6 +833,11 @@ class User extends Base
         $user->libraryQuota = $sanitizedParams->getInt('libraryQuota');
         $user->retired = $sanitizedParams->getCheckbox('retired');
 
+        // Are user home folders enabled? Don't change unless they are.
+        if ($this->getUser()->featureEnabled('folder.userHome')) {
+            $user->homeFolderId = $sanitizedParams->getInt('homeFolderId');
+        }
+
         if ($this->getUser()->isSuperAdmin()) {
             $user->userTypeId = $sanitizedParams->getInt('userTypeId');
             if ($user->retired === 1) {
@@ -1853,7 +1886,12 @@ class User extends Base
             throw new AccessDeniedException(__('This object is not shared with you with edit permission'));
         }
 
+        if ($object->permissionsClass() === 'Xibo\Entity\Folder' && $object->getId() === 1) {
+            throw new InvalidArgumentException(__('You cannot share the root folder'), 'id');
+        }
+
         $sanitizedParams = $this->getSanitizer($request->getParams());
+
         // Get all current permissions
         $permissions = $this->permissionFactory->getAllByObjectId($this->getUser(), $object->permissionsClass(), $id);
 
@@ -1872,7 +1910,7 @@ class User extends Base
 
             if ($object->canChangeOwner()) {
                 $object->setOwner($ownerId);
-                $object->save(['notify' => false, 'manageDynamicDisplayLinks' => false]);
+                $object->save(['notify' => false, 'manageDynamicDisplayLinks' => false, 'validate' => false]);
             } else {
                 throw new ConfigurationException(__('Cannot change owner on this Object'));
             }
@@ -2003,16 +2041,19 @@ class User extends Base
         if ($objectId == 0) {
             throw new InvalidArgumentException(__('Sharing form requested without an object'));
         }
-        
+
         /** @var ParsePermissionEntityEvent $event */
-        $event = $this->getDispatcher()->dispatch(ParsePermissionEntityEvent::$NAME . lcfirst($entity), new ParsePermissionEntityEvent($entity, $objectId));
+        $event = $this->getDispatcher()->dispatch(
+            new ParsePermissionEntityEvent($entity, $objectId),
+            ParsePermissionEntityEvent::$NAME . lcfirst($entity)
+        );
 
         return $event->getObject();
     }
 
     /**
      * Updates a set of permissions from a set of groupIds
-     * @param array[Permission] $permissions
+     * @param Permission[] $permissions
      * @param array $groupIds
      */
     private function updatePermissions($permissions, $groupIds)
@@ -2021,8 +2062,6 @@ class User extends Base
 
         // List of groupIds with view, edit and del assignments
         foreach ($permissions as $row) {
-            /* @var \Xibo\Entity\Permission $row */
-
             // Check and see what permissions we have been provided for this selection
             // If all permissions are 0, then the record is deleted
             if (is_array($groupIds)) {
@@ -2368,6 +2407,12 @@ class User extends Base
      *      required=false,
      *      type="integer"
      *   ),
+     *     @SWG\Parameter(
+     *      name="rememberFolderTreeStateGlobally",
+     *      in="formData",
+     *      required=false,
+     *      type="integer"
+     *   ),
      *   @SWG\Response(
      *      response=204,
      *      description="successful operation"
@@ -2389,6 +2434,7 @@ class User extends Base
         $this->getUser()->setOptionValue('useLibraryDuration', $parsedParams->getCheckbox('useLibraryDuration'));
         $this->getUser()->setOptionValue('showThumbnailColumn', $parsedParams->getCheckbox('showThumbnailColumn'));
         $this->getUser()->setOptionValue('isAlwaysUseManualAddUserForm', $parsedParams->getCheckbox('isAlwaysUseManualAddUserForm'));
+        $this->getUser()->setOptionValue('rememberFolderTreeStateGlobally', $parsedParams->getCheckbox('rememberFolderTreeStateGlobally'));
 
         if ($this->getUser()->isSuperAdmin()) {
             $this->getUser()->showContentFrom = $parsedParams->getInt('showContentFrom');
@@ -2440,4 +2486,70 @@ class User extends Base
 
         return $this->render($request, $response);
     }
+
+    /**
+     * Set home folder form
+     * @param Request $request
+     * @param Response $response
+     * @param $id
+     * @return \Psr\Http\Message\ResponseInterface|Response
+     * @throws GeneralException
+     * @throws \Xibo\Support\Exception\GeneralException
+     */
+    public function setHomeFolderForm(Request $request, Response $response, $id)
+    {
+        $user = $this->userFactory->getById($id);
+        $user->setChildAclDependencies($this->userGroupFactory);
+
+        if (!$this->getUser()->checkEditable($user)) {
+            throw new AccessDeniedException();
+        }
+
+        $this->getState()->template = 'user-form-home-folder';
+        $this->getState()->setData([
+            'user' => $user
+        ]);
+
+        return $this->render($request, $response);
+    }
+
+    /**
+     * Set home folder form
+     * @param Request $request
+     * @param Response $response
+     * @param $id
+     * @return \Psr\Http\Message\ResponseInterface|Response
+     * @throws GeneralException
+     * @throws \Xibo\Support\Exception\GeneralException
+     */
+    public function setHomeFolder(Request $request, Response $response, $id)
+    {
+        $user = $this->userFactory->getById($id);
+        $user->setChildAclDependencies($this->userGroupFactory);
+
+        if (!$this->getUser()->checkEditable($user)) {
+            throw new AccessDeniedException();
+        }
+
+        if (!$this->getUser()->featureEnabled('folder.userHome')) {
+            throw new AccessDeniedException();
+        }
+
+        $sanitizedParams = $this->getSanitizer($request->getParams());
+
+        // Build a user entity and save it
+        $user->setChildAclDependencies($this->userGroupFactory);
+        $user->load();
+        $user->homeFolderId = $sanitizedParams->getInt('homeFolderId');
+        $user->save();
+
+        // Return
+        $this->getState()->hydrate([
+            'message' => sprintf(__('Edited %s'), $user->userName),
+            'id' => $user->userId,
+            'data' => $user
+        ]);
+
+        return $this->render($request, $response);
+    }
 }
