@@ -200,6 +200,18 @@ class Media implements \JsonSerializable
      */
     public $orientation;
 
+    /**
+     * @var int
+     * @SWG\Property(description="The width of the Media file")
+     */
+    public $width;
+
+    /**
+     * @var int
+     * @SWG\Property(description="The height of the Media file")
+     */
+    public $height;
+
     // Private
     private $unassignTags = [];
     private $requestOptions = [];
@@ -622,8 +634,8 @@ class Media implements \JsonSerializable
         }
 
         $this->mediaId = $this->getStore()->insert('
-            INSERT INTO `media` (`name`, `type`, duration, originalFilename, userID, retired, moduleSystemFile, released, apiRef, valid, `createdDt`, `modifiedDt`, `enableStat`, `folderId`, `permissionsFolderId`, `orientation`)
-              VALUES (:name, :type, :duration, :originalFileName, :userId, :retired, :moduleSystemFile, :released, :apiRef, :valid, :createdDt, :modifiedDt, :enableStat, :folderId, :permissionsFolderId, :orientation)
+            INSERT INTO `media` (`name`, `type`, duration, originalFilename, userID, retired, moduleSystemFile, released, apiRef, valid, `createdDt`, `modifiedDt`, `enableStat`, `folderId`, `permissionsFolderId`, `orientation`, `width`, `height`)
+              VALUES (:name, :type, :duration, :originalFileName, :userId, :retired, :moduleSystemFile, :released, :apiRef, :valid, :createdDt, :modifiedDt, :enableStat, :folderId, :permissionsFolderId, :orientation, :width, :height)
         ', [
             'name' => $this->name,
             'type' => $this->mediaType,
@@ -640,7 +652,9 @@ class Media implements \JsonSerializable
             'enableStat' => $this->enableStat,
             'folderId' => ($this->folderId === null) ? 1 : $this->folderId,
             'permissionsFolderId' => ($this->permissionsFolderId == null) ? 1 : $this->permissionsFolderId,
-            'orientation' => $this->orientation
+            'orientation' => $this->orientation,
+            'width' => ($this->width === null) ? null : $this->width,
+            'height' => ($this->height === null) ? null : $this->height,
         ]);
     }
 
@@ -665,7 +679,9 @@ class Media implements \JsonSerializable
                 expires = :expires,
                 folderId = :folderId,
                 permissionsFolderId = :permissionsFolderId,
-                orientation = :orientation
+                orientation = :orientation,
+                width = :width,
+                height = :height
            WHERE mediaId = :mediaId
         ';
 
@@ -685,7 +701,9 @@ class Media implements \JsonSerializable
             'expires' => $this->expires,
             'folderId' => $this->folderId,
             'permissionsFolderId' => $this->permissionsFolderId,
-            'orientation' => $this->orientation
+            'orientation' => $this->orientation,
+            'width' => ($this->width === null) ? null : $this->width,
+            'height' => ($this->height === null) ? null : $this->height,
         ];
 
         $this->getStore()->update($sql, $params);
@@ -800,6 +818,8 @@ class Media implements \JsonSerializable
                     expires = :expires,
                     released = :released,
                     orientation = :orientation,
+                    width = :width,
+                    height = :height,
                     valid = :valid 
              WHERE mediaId = :mediaId
         ', [
@@ -810,7 +830,9 @@ class Media implements \JsonSerializable
             'released' => $this->released,
             'orientation' => $this->orientation,
             'valid' => $this->valid,
-            'mediaId' => $this->mediaId
+            'width' => ($this->width === null) ? null : $this->width,
+            'height' => ($this->height === null) ? null : $this->height,
+            'mediaId' => $this->mediaId,
         ]);
     }
 
@@ -824,6 +846,8 @@ class Media implements \JsonSerializable
             $resizeThreshold = $this->config->getSetting('DEFAULT_RESIZE_THRESHOLD');
             $resizeLimit = $this->config->getSetting('DEFAULT_RESIZE_LIMIT');
 
+            $this->width = $imgWidth;
+            $this->height = $imgHeight;
             $this->orientation = ($imgWidth >= $imgHeight) ? 'landscape' : 'portrait';
 
             // Media released set to 0 for large size images
@@ -986,21 +1010,29 @@ class Media implements \JsonSerializable
 
     /**
      * Update Media orientation.
-     * For videos from Library connectors, update the orientation once we have the the cover image saved.
-     * @param string $orientation
+     * For videos from Library connectors, update the orientation once we have the cover image saved.
+     * @param int $width
+     * @param int $height
      * @return Media
      */
-    public function updateOrientation(string $orientation): Media
+    public function updateOrientation(int $width, int $height): Media
     {
-        $this->getLog()->debug('Updating duration for MediaId '. $this->mediaId);
+        $this->getLog()->debug('Updating orientation and resolution for MediaId '. $this->mediaId);
 
-        $this->getStore()->update('UPDATE `media` SET orientation = :orientation WHERE mediaId = :mediaId', [
-            'orientation' => $orientation,
+        $this->width = $width;
+        $this->height = $height;
+        $this->orientation = ($width >= $height) ? 'landscape' : 'portrait';
+
+        $this->getStore()->update('
+            UPDATE `media` SET orientation = :orientation, width = :width, height = :height
+             WHERE mediaId = :mediaId
+        ', [
+            'orientation' => $this->orientation,
+            'width' => $this->width,
+            'height' => $this->height,
             'mediaId' => $this->mediaId
         ]);
 
-        $this->orientation = $orientation;
-
         return $this;
     }
 }
