@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - http://www.xibo.org.uk
  *
@@ -84,6 +84,12 @@ class Playlist implements \JsonSerializable
      */
     public $filterMediaName;
 
+    /**
+     * @SWG\Property(description="Which logical operator should be used when filtering by multiple Plalust names? OR|AND")
+     * @var string
+     */
+    public $filterMediaNameLogicalOperator;
+
     /**
      * @SWG\Property(description="Filter Tags for a Dynamic Playlist")
      * @var string
@@ -100,7 +106,7 @@ class Playlist implements \JsonSerializable
      * @SWG\Property(description="Which logical operator should be used when filtering by multiple Tags? OR|AND")
      * @var string
      */
-    public $filterLogicalOperator;
+    public $filterMediaTagsLogicalOperator;
 
     /**
      * @SWG\Property(description="Maximum number of Media items matching dynamic Playlist filters")
@@ -734,9 +740,12 @@ class Playlist implements \JsonSerializable
             $this->load();
         }
 
-        if (!$options['regionDelete'] && $this->regionId != 0)
-            throw new InvalidArgumentException(__('This Playlist belongs to a Region, please delete the Region instead.'), 'regionId');
-
+        if (!$options['regionDelete'] && $this->regionId != 0) {
+            throw new InvalidArgumentException(
+                __('This Playlist belongs to a Region, please delete the Region instead.'),
+                'regionId'
+            );
+        }
         // Notify we're going to delete
         // we do this here, because once we've deleted we lose the references for the storage query
         $this->notifyLayouts();
@@ -749,22 +758,50 @@ class Playlist implements \JsonSerializable
             $parent->load();
             foreach ($parent->widgets as $widget) {
                 if ($widget->type === 'subplaylist') {
+                    $isWidgetSaveRequired = false;
+                    /** @var $module SubPlaylist **/
+                    $module = $this->moduleFactory->createWithWidget($widget);
                     // we get an array with all subplaylists assigned to the parent
-                    $subPlaylistIds = json_decode($widget->getOptionValue('subPlaylistIds', '[]'));
-                    foreach ($subPlaylistIds as $subplaylist) {
+                    $subPlaylistItems = $module->getAssignedPlaylists();
+                    $i = 0;
+                    foreach ($subPlaylistItems as $subPlaylistItem) {
                         // find the matching playlistId to the playlistId we want to delete
-                        if ($subplaylist == $this->playlistId) {
-                            // if there is only one element in the subPlaylistIds array then remove the widget
-                            if (count($subPlaylistIds) === 1) {
+                        if ($subPlaylistItem->playlistId == $this->playlistId) {
+                            // if there is only one playlistItem in subPlaylists option then remove the widget
+                            if (count($subPlaylistItems) === 1) {
                                 $widget->delete(['notify' => false]);
                             } else {
-                                // if the subPlaylistIds has more than one element, we want to just unassign our playlistId from it and save the widget,
+                                // if we have more than one subPlaylist item in subPlaylists option,
+                                // we want to just unassign our playlist from it and save the widget,
                                 // we don't want to remove the whole widget in this case
-                                $updatedSubplaylistIds = array_diff($subPlaylistIds, [$this->playlistId]);
-                                $widget->setOptionValue('subPlaylistIds', 'attrib', json_encode($updatedSubplaylistIds));
-                                $widget->save();
+                                unset($subPlaylistItems[$i]);
+                                $isWidgetSaveRequired = true;
                             }
                         }
+                        $i++;
+                    }
+
+                    if ($isWidgetSaveRequired) {
+                        // update row numbers for each element
+                        // from getAssignedPlaylists the spots/length will come as null/int
+                        // make sure spots and spotLength are saved as string if empty
+                        $j = 1;
+                        foreach ($subPlaylistItems as $subPlaylistItem) {
+                            $subPlaylistItem->rowNo = $j;
+                            $subPlaylistItem->spots = $subPlaylistItem->spots ?? '';
+                            $subPlaylistItem->spotLength = $subPlaylistItem->spotLength ?? '';
+                            $j++;
+                        }
+
+                        // update subPlaylists Widget option
+                        $widget->setOptionValue('subPlaylists', 'attrib', json_encode(array_values($subPlaylistItems)));
+
+                        // Tidy up any old options
+                        if ($module->getOption('subPlaylistIds') !== null) {
+                            $widget->setOptionValue('subPlaylistIds', 'attrib', null);
+                            $widget->setOptionValue('subPlaylistOptions', 'attrib', null);
+                        }
+                        $widget->save();
                     }
                 }
             }
@@ -814,8 +851,8 @@ class Playlist implements \JsonSerializable
         $time = Carbon::now()->format(DateFormatHelper::getSystemFormat());
 
         $sql = '
-        INSERT INTO `playlist` (`name`, `ownerId`, `regionId`, `isDynamic`, `filterMediaName`, `filterMediaTags`, `filterExactTags`, `filterLogicalOperator`, `maxNumberOfItems`, `createdDt`, `modifiedDt`, `requiresDurationUpdate`, `enableStat`, `folderId`, `permissionsFolderId`) 
-          VALUES (:name, :ownerId, :regionId, :isDynamic, :filterMediaName, :filterMediaTags, :filterExactTags, :filterLogicalOperator, :maxNumberOfItems, :createdDt, :modifiedDt, :requiresDurationUpdate, :enableStat, :folderId, :permissionsFolderId)
+        INSERT INTO `playlist` (`name`, `ownerId`, `regionId`, `isDynamic`, `filterMediaName`, `filterMediaNameLogicalOperator`, `filterMediaTags`, `filterExactTags`, `filterMediaTagsLogicalOperator`, `maxNumberOfItems`, `createdDt`, `modifiedDt`, `requiresDurationUpdate`, `enableStat`, `folderId`, `permissionsFolderId`) 
+          VALUES (:name, :ownerId, :regionId, :isDynamic, :filterMediaName, :filterMediaNameLogicalOperator, :filterMediaTags, :filterExactTags, :filterMediaTagsLogicalOperator, :maxNumberOfItems, :createdDt, :modifiedDt, :requiresDurationUpdate, :enableStat, :folderId, :permissionsFolderId)
         ';
         $this->playlistId = $this->getStore()->insert($sql, array(
             'name' => $this->name,
@@ -823,9 +860,10 @@ class Playlist implements \JsonSerializable
             'regionId' => $this->regionId == 0 ? null : $this->regionId,
             'isDynamic' => $this->isDynamic,
             'filterMediaName' => $this->filterMediaName,
+            'filterMediaNameLogicalOperator' => $this->filterMediaNameLogicalOperator ?? 'OR',
             'filterMediaTags' => $this->filterMediaTags,
             'filterExactTags' => $this->filterExactTags ?? 0,
-            'filterLogicalOperator' => $this->filterLogicalOperator ?? 'OR',
+            'filterMediaTagsLogicalOperator' => $this->filterMediaTagsLogicalOperator ?? 'OR',
             'maxNumberOfItems' => $this->isDynamic == 0 ? null : $this->maxNumberOfItems,
             'createdDt' => $time,
             'modifiedDt' => $time,
@@ -858,9 +896,10 @@ class Playlist implements \JsonSerializable
                 `duration` = :duration,
                 `isDynamic` = :isDynamic,
                 `filterMediaName` = :filterMediaName,
+                `filterMediaNameLogicalOperator` = :filterMediaNameLogicalOperator,
                 `filterMediaTags` = :filterMediaTags,
                 `filterExactTags` = :filterExactTags,
-                `filterLogicalOperator` = :filterLogicalOperator,
+                `filterMediaTagsLogicalOperator` = :filterMediaTagsLogicalOperator,
                 `maxNumberOfItems` = :maxNumberOfItems,
                 `requiresDurationUpdate` = :requiresDurationUpdate,
                 `enableStat` = :enableStat,
@@ -877,9 +916,10 @@ class Playlist implements \JsonSerializable
             'duration' => $this->duration,
             'isDynamic' => $this->isDynamic,
             'filterMediaName' => $this->filterMediaName,
+            'filterMediaNameLogicalOperator' => $this->filterMediaNameLogicalOperator ?? 'OR',
             'filterMediaTags' => $this->filterMediaTags,
             'filterExactTags' => $this->filterExactTags ?? 0,
-            'filterLogicalOperator' => $this->filterLogicalOperator ?? 'OR',
+            'filterMediaTagsLogicalOperator' => $this->filterMediaTagsLogicalOperator ?? 'OR',
             'maxNumberOfItems' => $this->maxNumberOfItems,
             'modifiedDt' => Carbon::now()->format(DateFormatHelper::getSystemFormat()),
             'requiresDurationUpdate' => $this->requiresDurationUpdate,
@@ -1127,7 +1167,7 @@ class Playlist implements \JsonSerializable
      * playlistMappings, nestedPLaylistDefinitions, dataSets and dataSetIds are passed by reference.
      *
      *
-     * @param $widgets array An array of widgets assigned to the Playlist
+     * @param $widgets Widget[] An array of widgets assigned to the Playlist
      * @param $parentId int Playlist Id of the Playlist that is a parent to our current Playlist
      * @param $playlistMappings array An array of Playlists with ParentId and PlaylistId as keys
      * @param $count
@@ -1141,54 +1181,59 @@ class Playlist implements \JsonSerializable
      */
     public function generatePlaylistMapping($widgets, $parentId, &$playlistMappings, &$count, &$nestedPlaylistDefinitions, &$dataSetIds, &$dataSets, $dataSetFactory, $includeData)
     {
-            foreach ($widgets as $playlistWidget) {
+        foreach ($widgets as $playlistWidget) {
+            if ($playlistWidget->type == 'subplaylist') {
+                $playlistItems = json_decode($playlistWidget->getOptionValue('subPlaylists', '[]'), true);
+                foreach ($playlistItems as $nestedPlaylistItem) {
+                    $nestedPlaylist = $this->playlistFactory->getById($nestedPlaylistItem['playlistId']);
+                    $nestedPlaylist->load();
+                    $this->getLog()->debug('playlist mappings parent id ' . $parentId);
+                    $nestedPlaylistDefinitions[$nestedPlaylist->playlistId] = $nestedPlaylist;
 
-                if ($playlistWidget->type == 'subplaylist') {
+                    $playlistMappings[$parentId][$nestedPlaylist->playlistId] = [
+                        'parentId' => $parentId,
+                        'playlist' => $nestedPlaylist->name,
+                        'playlistId' => $nestedPlaylist->playlistId
+                    ];
 
-                    $nestedPlaylistIds = json_decode($playlistWidget->getOptionValue('subPlaylistIds', []), true);
-                    foreach ($nestedPlaylistIds as $nestedPlaylistId) {
-                        $nestedPlaylist = $this->playlistFactory->getById($nestedPlaylistId);
-                        $nestedPlaylist->load();
-                        $this->getLog()->debug('playlist mappings parent id ' . $parentId);
-                        $nestedPlaylistDefinitions[$nestedPlaylist->playlistId] = $nestedPlaylist;
+                    $count++;
 
-                        $playlistMappings[$parentId][$nestedPlaylist->playlistId] = [
-                            'parentId' => $parentId,
-                            'playlist' => $nestedPlaylist->name,
-                            'playlistId' => $nestedPlaylist->playlistId
-                        ];
-
-                        $count++;
-
-                        // this is a recursive function, we need to go through all levels of nested Playlists.
-                        $this->generatePlaylistMapping($nestedPlaylist->widgets, $nestedPlaylist->playlistId, $playlistMappings, $count, $nestedPlaylistDefinitions,$dataSetIds, $dataSets, $dataSetFactory, $includeData);
-                    }
-                }
-
-                // if we have any widgets that use DataSets we want the dataSetId and data added
-                if ($playlistWidget->type == 'datasetview' || $playlistWidget->type == 'datasetticker' || $playlistWidget->type == 'chart') {
-                    $dataSetId = $playlistWidget->getOptionValue('dataSetId', 0);
-
-                    if ($dataSetId != 0) {
-
-                        if (in_array($dataSetId, $dataSetIds))
-                            continue;
-
-                        // Export the structure for this dataSet
-                        $dataSet = $dataSetFactory->getById($dataSetId);
-                        $dataSet->load();
-
-                        // Are we also looking to export the data?
-                        if ($includeData) {
-                            $dataSet->data = $dataSet->getData([], ['includeFormulaColumns' => false]);
-                        }
-
-                        $dataSetIds[] = $dataSet->dataSetId;
-                        $dataSets[] = $dataSet;
-                    }
+                    // this is a recursive function, we need to go through all levels of nested Playlists.
+                    $this->generatePlaylistMapping(
+                        $nestedPlaylist->widgets,
+                        $nestedPlaylist->playlistId,
+                        $playlistMappings,
+                        $count,
+                        $nestedPlaylistDefinitions,
+                        $dataSetIds,
+                        $dataSets,
+                        $dataSetFactory,
+                        $includeData
+                    );
                 }
             }
+            // if we have any widgets that use DataSets we want the dataSetId and data added
+            if ($playlistWidget->type == 'datasetview' || $playlistWidget->type == 'datasetticker' || $playlistWidget->type == 'chart') {
+                $dataSetId = $playlistWidget->getOptionValue('dataSetId', 0);
+                if ($dataSetId != 0) {
+                    if (in_array($dataSetId, $dataSetIds)) {
+                        continue;
+                    }
+                    // Export the structure for this dataSet
+                    $dataSet = $dataSetFactory->getById($dataSetId);
+                    $dataSet->load();
 
-            return $playlistMappings;
+                    // Are we also looking to export the data?
+                    if ($includeData) {
+                        $dataSet->data = $dataSet->getData([], ['includeFormulaColumns' => false]);
+                    }
+
+                    $dataSetIds[] = $dataSet->dataSetId;
+                    $dataSets[] = $dataSet;
+                }
+            }
+        }
+
+        return $playlistMappings;
     }
 }
