@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -56,6 +56,10 @@ use Xibo\Support\Exception\NotFoundException;
  * @package Xibo\Entity
  *
  * @SWG\Definition()
+ * @property $libraryQuotaFormatted
+ * @property $twoFactorDescription
+ * @property $homePage
+ * @property $homeFolder
  */
 class User implements \JsonSerializable, UserEntityInterface
 {
@@ -97,6 +101,12 @@ class User implements \JsonSerializable, UserEntityInterface
      */
     public $homePageId;
 
+    /**
+     * @SWG\Property(description="This users home folder")
+     * @var int
+     */
+    public $homeFolderId;
+
     /**
      * @SWG\Property(description="A timestamp indicating the time the user last logged into the CMS")
      * @var int
@@ -754,7 +764,10 @@ class User implements \JsonSerializable, UserEntityInterface
         } else if ($options['passwordUpdate']) {
             $this->updatePassword();
             $this->audit($this->userId, 'User updated password', false);
-        } else if ($this->hash() != $this->hash || $this->hasPropertyChanged('twoFactorRecoveryCodes')) {
+        } else if ($this->hash() != $this->hash
+            || $this->hasPropertyChanged('twoFactorRecoveryCodes')
+            || $this->hasPropertyChanged('password')
+        ) {
             $this->update();
             $this->audit($this->userId, 'User updated');
         }
@@ -813,8 +826,8 @@ class User implements \JsonSerializable, UserEntityInterface
      */
     private function add()
     {
-        $sql = 'INSERT INTO `user` (UserName, UserPassword, isPasswordChangeRequired, usertypeid, newUserWizard, email, homePageId, CSPRNG, firstName, lastName, phone, ref1, ref2, ref3, ref4, ref5)
-                     VALUES (:userName, :password, :isPasswordChangeRequired, :userTypeId, :newUserWizard, :email, :homePageId, :CSPRNG, :firstName, :lastName, :phone, :ref1, :ref2, :ref3, :ref4, :ref5)';
+        $sql = 'INSERT INTO `user` (UserName, UserPassword, isPasswordChangeRequired, usertypeid, newUserWizard, email, homePageId, homeFolderId, CSPRNG, firstName, lastName, phone, ref1, ref2, ref3, ref4, ref5)
+                     VALUES (:userName, :password, :isPasswordChangeRequired, :userTypeId, :newUserWizard, :email, :homePageId, :homeFolderId, :CSPRNG, :firstName, :lastName, :phone, :ref1, :ref2, :ref3, :ref4, :ref5)';
 
         // Get the ID of the record we just inserted
         $this->userId = $this->getStore()->insert($sql, [
@@ -825,6 +838,7 @@ class User implements \JsonSerializable, UserEntityInterface
             'newUserWizard' => $this->newUserWizard,
             'email' => $this->email,
             'homePageId' => $this->homePageId,
+            'homeFolderId' => $this->homeFolderId,
             'CSPRNG' => $this->CSPRNG,
             'firstName' => $this->firstName,
             'lastName' => $this->lastName,
@@ -860,6 +874,7 @@ class User implements \JsonSerializable, UserEntityInterface
 
         $sql = 'UPDATE `user` SET UserName = :userName,
                   homePageId = :homePageId,
+                  homeFolderId = :homeFolderId,
                   Email = :email,
                   Retired = :retired,
                   userTypeId = :userTypeId,
@@ -886,6 +901,7 @@ class User implements \JsonSerializable, UserEntityInterface
             'userTypeId' => $this->userTypeId,
             'email' => $this->email,
             'homePageId' => $this->homePageId,
+            'homeFolderId' => $this->homeFolderId,
             'retired' => $this->retired,
             'newUserWizard' => $this->newUserWizard,
             'CSPRNG' => $this->CSPRNG,
@@ -1038,9 +1054,8 @@ class User implements \JsonSerializable, UserEntityInterface
             // Store the results in the cache (default to empty result)
             $this->permissionCache[$entity] = array();
 
-            // Turn it into a ID keyed array
+            // Turn it into an ID keyed array
             foreach ($this->permissionFactory->getByUserId($entity, $this->userId) as $permission) {
-                /* @var \Xibo\Entity\Permission $permission */
                 // Always take the max
                 if (array_key_exists($permission->objectId, $this->permissionCache[$entity])) {
                     $old = $this->permissionCache[$entity][$permission->objectId];
@@ -1051,9 +1066,9 @@ class User implements \JsonSerializable, UserEntityInterface
                     $new->delete = max($permission->delete, $old->delete);
 
                     $this->permissionCache[$entity][$permission->objectId] = $new;
-                }
-                else
+                } else {
                     $this->permissionCache[$entity][$permission->objectId] = $permission;
+                }
             }
         }
 
@@ -1127,6 +1142,13 @@ class User implements \JsonSerializable, UserEntityInterface
         $permissions = $this->loadPermissions($object->permissionsClass());
         $folderPermissions = $this->loadPermissions('Xibo\Entity\Folder');
 
+        // If we are checking for view permissions on a folder, then we always grant those to a users home folder.
+        if ($object->permissionsClass() === 'Xibo\Entity\Folder'
+            && $object->getId() === $this->homeFolderId
+        ) {
+            return true;
+        }
+
         // Check to see if our object is in the list
         if (array_key_exists($object->getId(), $permissions)) {
             return ($permissions[$object->getId()]->view == 1);
@@ -1153,9 +1175,18 @@ class User implements \JsonSerializable, UserEntityInterface
             return true;
 
         // Group Admins
-        if ($this->userTypeId == 2 && count(array_intersect($this->groups, $this->userGroupFactory->getByUserId($object->getOwnerId()))))
-            // Group Admin and in the same group as the owner.
-            return true;
+        if ($object->permissionsClass() === 'Xibo\Entity\UserGroup') {
+            // userGroup does not have an owner (getOwnerId() returns 0 ), we need to handle it in a different way.
+            if ($this->userTypeId == 2 && count(array_intersect($this->groups, [$object]))) {
+                // Group Admin and group object in the user array of groups
+                return true;
+            }
+        } else {
+            if ($this->userTypeId == 2 && count(array_intersect($this->groups, $this->userGroupFactory->getByUserId($object->getOwnerId())))) {
+                // Group Admin and in the same group as the owner.
+                return true;
+            }
+        }
 
         // Get the permissions for that entity
         $permissions = $this->loadPermissions($object->permissionsClass());
@@ -1299,7 +1330,7 @@ class User implements \JsonSerializable, UserEntityInterface
                   INNER JOIN `lkusergroup`
                   ON lkusergroup.userId = media.userId
                WHERE lkusergroup.groupId = :groupId
-            ', ['groupId' => $groupId], null, true);
+            ', ['groupId' => $groupId], 'default', $reconnect);
 
             if (count($rows) <= 0) {
                 throw new LibraryFullException("Error Processing Request", 1);
