@@ -1,6 +1,6 @@
 <?php
-/**
- * Copyright (C) 2020 Xibo Signage Ltd
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - http://www.xibo.org.uk
  *
@@ -24,15 +24,19 @@
 namespace Xibo\XTR;
 use Carbon\Carbon;
 use Xibo\Controller\Display;
+use Xibo\Event\DisplayGroupLoadEvent;
 use Xibo\Event\MaintenanceRegularEvent;
 use Xibo\Factory\DisplayFactory;
+use Xibo\Factory\DisplayGroupFactory;
 use Xibo\Factory\LayoutFactory;
 use Xibo\Factory\ModuleFactory;
 use Xibo\Factory\NotificationFactory;
 use Xibo\Factory\PlaylistFactory;
+use Xibo\Factory\ScheduleFactory;
 use Xibo\Factory\UserGroupFactory;
 use Xibo\Helper\ByteFormatter;
 use Xibo\Helper\DateFormatHelper;
+use Xibo\Helper\Profiler;
 use Xibo\Helper\WakeOnLan;
 use Xibo\Service\MediaServiceInterface;
 use Xibo\Support\Exception\GeneralException;
@@ -55,6 +59,9 @@ class MaintenanceRegularTask implements TaskInterface
     /** @var DisplayFactory */
     private $displayFactory;
 
+    /** @var DisplayGroupFactory */
+    private $displayGroupFactory;
+
     /** @var NotificationFactory */
     private $notificationFactory;
 
@@ -72,6 +79,10 @@ class MaintenanceRegularTask implements TaskInterface
 
     /** @var  \Xibo\Helper\SanitizerService */
     private $sanitizerService;
+    /**
+     * @var ScheduleFactory
+     */
+    private $scheduleFactory;
 
     /** @inheritdoc */
     public function setFactories($container)
@@ -80,12 +91,14 @@ class MaintenanceRegularTask implements TaskInterface
         $this->mediaService = $container->get('mediaService');
 
         $this->displayFactory = $container->get('displayFactory');
+        $this->displayGroupFactory = $container->get('displayGroupFactory');
         $this->notificationFactory = $container->get('notificationFactory');
         $this->userGroupFactory = $container->get('userGroupFactory');
         $this->layoutFactory = $container->get('layoutFactory');
         $this->playlistFactory = $container->get('playlistFactory');
         $this->moduleFactory = $container->get('moduleFactory');
         $this->sanitizerService = $container->get('sanitizerService');
+        $this->scheduleFactory = $container->get('scheduleFactory');
         return $this;
     }
 
@@ -112,6 +125,10 @@ class MaintenanceRegularTask implements TaskInterface
 
         $this->publishLayouts();
 
+        $this->assessDynamicDisplayGroups();
+
+        $this->tidyAdCampaignSchedules();
+
         // Dispatch an event so that consumers can hook into regular maintenance.
         $event = new MaintenanceRegularEvent();
         $this->getDispatcher()->dispatch($event, MaintenanceRegularEvent::$NAME);
@@ -415,7 +432,9 @@ class MaintenanceRegularTask implements TaskInterface
         if (count($layouts) > 0) {
             foreach ($layouts as $layout) {
                 // check if the layout should be published now according to the date
-                if (Carbon::createFromTimestamp($layout->publishedDate)->format('U') < Carbon::now()->format('U')) {
+                if (Carbon::createFromFormat(DateFormatHelper::getSystemFormat(), $layout->publishedDate)
+                    ->isBefore(Carbon::now()->format(DateFormatHelper::getSystemFormat()))
+                ) {
                     try {
                         // publish the layout
                         $layout = $this->layoutFactory->concurrentRequestLock($layout, true);
@@ -471,4 +490,70 @@ class MaintenanceRegularTask implements TaskInterface
 
         $this->runMessage .= ' - Done' . PHP_EOL . PHP_EOL;
     }
-}
\ No newline at end of file
+
+    /**
+     * Assess any eligible dynamic display groups if necessary
+     * @return void
+     */
+    private function assessDynamicDisplayGroups()
+    {
+        $this->runMessage .= '## ' . __('Assess Dynamic Display Groups') . PHP_EOL;
+
+        if ($this->config->getSetting('DYNAMIC_DISPLAY_GROUP_ASSESS', 0) == 1) {
+            Profiler::start('RegularMaintenance::assessDynamicDisplayGroups', $this->log);
+
+            $this->config->changeSetting('DYNAMIC_DISPLAY_GROUP_ASSESS', 0);
+            $count = 0;
+
+            foreach ($this->displayGroupFactory->getByIsDynamic(1) as $group) {
+                $count++;
+                try {
+                    // Loads displays.
+                    $this->getDispatcher()->dispatch(
+                        new DisplayGroupLoadEvent($group),
+                        DisplayGroupLoadEvent::$NAME
+                    );
+                    $group->save([
+                        'validate' => false,
+                        'saveGroup' => false,
+                        'saveTags' => false,
+                        'manageLinks' => false,
+                        'manageDisplayLinks' => false,
+                        'manageDynamicDisplayLinks' => true,
+                        'allowNotify' => true
+                    ]);
+                } catch (GeneralException $exception) {
+                    $this->log->error('assessDynamicDisplayGroups: Unable to manage group: '
+                        . $group->displayGroup);
+                }
+            }
+            Profiler::end('RegularMaintenance::assessDynamicDisplayGroups', $this->log);
+            $this->runMessage .= ' - Done ' . $count . PHP_EOL . PHP_EOL;
+        } else {
+            $this->runMessage .= ' - Done (not required)' . PHP_EOL . PHP_EOL;
+        }
+    }
+
+    private function tidyAdCampaignSchedules()
+    {
+        $this->runMessage .= '## ' . __('Tidy Ad Campaign Schedules') . PHP_EOL;
+        Profiler::start('RegularMaintenance::tidyAdCampaignSchedules', $this->log);
+        $count = 0;
+
+        foreach ($this->scheduleFactory->query(null, [
+            'adCampaignsOnly' => 1,
+            'toDt' => Carbon::now()->subDays(90)->unix()
+        ]) as $event) {
+            if (!empty($event->parentCampaignId)) {
+                $count++;
+                $this->log->debug('tidyAdCampaignSchedules : Found old Ad Campaign interrupt event ID '
+                    . $event->eventId . ' deleting');
+                $event->delete(['notify' => false]);
+            }
+        }
+
+        $this->log->debug('tidyAdCampaignSchedules : Deleted ' . $count . ' events');
+        Profiler::end('RegularMaintenance::tidyAdCampaignSchedules', $this->log);
+        $this->runMessage .= ' - Done ' . $count . PHP_EOL . PHP_EOL;
+    }
+}
